package com.yunzhi.nanyang.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.nanyang.common.BaseController;
import com.yunzhi.nanyang.common.Constants;
import com.yunzhi.nanyang.common.ResponseBean;
import com.yunzhi.nanyang.common.StringUtils;
import com.yunzhi.nanyang.entity.SysUser;
import com.yunzhi.nanyang.entity.TaOrder;
import com.yunzhi.nanyang.entity.TaWorkJob;
import com.yunzhi.nanyang.service.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import com.yunzhi.nanyang.entity.TaDispatch;
import org.springframework.web.bind.annotation.RestController;

/**
 * <p>
    * 调度表 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2022-03-08
 */

@Api(tags = "调度表")
@RestController
@RequestMapping("/")
public class TaDispatchController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaDispatchController.class);

    @Autowired
    ITaOrgService iTaOrgService;

    @Autowired
    public ITaDispatchService iTaDispatchService;

    @Autowired
    public ITaOrderService iTaOrderService;

    @Autowired
    public ITaMachineryService iTaMachineryService;

    @Autowired
    public ITdMachineryTypeService iTdMachineryTypeService;

    @Autowired
    public ITaWorkJobService iTaWorkJobService;


//    /**
//     * 分页查询列表
//     * @param pageNum
//     * @param pageSize
//     * @return
//     */
//    @RequestMapping(value="/admin/dispatch",method= RequestMethod.GET)
//    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
//    public ResponseBean taDispatchList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
//									 @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize) throws Exception{
//
//        IPage<TaDispatch> pg = new Page<>(pageNum, pageSize);
//        QueryWrapper<TaDispatch> queryWrapper = new QueryWrapper<>();
//        queryWrapper.orderByDesc("create_date");
//
//        IPage<TaDispatch> result = iTaDispatchService.page(pg, queryWrapper);
//        return ResponseBean.success(result);
//    }

    /**
     * 保存对象
     * @param taDispatch 实体对象
     * @return
     */
    @RequestMapping(value="/admin/dispatch",method= RequestMethod.POST)
    @ApiOperation(value="保存", notes = "保存", httpMethod = "POST", response = ResponseBean.class)
    @RequiresPermissions("orderjob:add")
    public ResponseBean taDispatchAdd(@ApiParam("保存内容") @RequestBody TaDispatch taDispatch) throws Exception{
        SysUser user = currentUser();

        taDispatch.setDispatchId(null);

        if (StringUtils.isEmpty(taDispatch.getOrderId()) || StringUtils.isEmpty(taDispatch.getOrderId())) {
            return ResponseBean.error("订单或者接单机构为空");
        }
        TaOrder taOrder = iTaOrderService.getExistBy("order_id", taDispatch.getOrderId(), false, true);
        if (null == taOrder) {
            return ResponseBean.error("验证订单信息异常");
        }

        if (!checkOrgAccess(taDispatch.getOrgId(), user.getUserId())) {
            return ResponseBean.error("无操作权限");
        }

        taDispatch.setOrgName(iTaOrgService.getById(taDispatch.getOrgId()).getName());

        if (!StringUtils.isEmpty(taDispatch.getMachineryId())) {
            taDispatch.setMachineryName(iTaMachineryService.getById(taDispatch.getMachineryId()).getName());
        }

        if (!StringUtils.isEmpty(taDispatch.getTypeId())) {
            taDispatch.setTypeName(iTdMachineryTypeService.getById(taDispatch.getTypeId()).getName());
        }

        taDispatch.setStatus(Constants.STATUS_NORMAL);


        if (iTaDispatchService.save(taDispatch)){

            // 生成作业信息
            iTaWorkJobService.createByDispatch(taDispatch, taOrder);

            // 更新订单状态
            iTaOrderService.updateStatus(taDispatch.getOrderId(), "dispatch_status", Constants.DISPATCH_DONE);

            return ResponseBean.success(taDispatch);
        }else {
            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/dispatch/{id}", method= RequestMethod.DELETE)
    @ApiOperation(value="删除", notes = "删除", httpMethod = "DELETE", response = ResponseBean.class)
    @RequiresPermissions("orderjob:delete")
    public ResponseBean taDispatchDelete(@ApiParam("对象ID") @PathVariable String id) throws Exception{
        TaDispatch taDispatch = iTaDispatchService.getById(id);

        SysUser user = currentUser();

        if (!checkOrgAccess(taDispatch.getOrgId(), user.getUserId())) {
            return ResponseBean.error("无操作权限");
        }

        TaWorkJob taWorkJob = iTaWorkJobService.getExistBy("dispatch_id", id, false, true);
        if (null == taWorkJob) {
            return ResponseBean.error("校验作业信息失败");
        }

//        if (Constants.WORK_READY != taWorkJob.getStatus()) {
//            return ResponseBean.error("作业已开始，不能删除");
//        }

        // 先清除原来的作业信息
        iTaWorkJobService.removeLogicById(taWorkJob.getJobId());

        if(iTaDispatchService.removeLogicById(id)){
            // 更新订单状态
            iTaOrderService.updateStatus(taDispatch.getOrderId(), "dispatch_status", Constants.DISPATCH_READY);
            return ResponseBean.success("success");
        } else {
            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

//    /**
//     * 修改对象
//     * @param id  实体ID
//     * @param taDispatch 实体对象
//     * @return
//     */
//    @RequestMapping(value="/taDispatch/{id}",method= RequestMethod.PUT)
//    @ApiOperation(value="更新", notes = "更新", httpMethod = "PUT", response = ResponseBean.class)
//    public ResponseBean taDispatchUpdate(@ApiParam("对象ID") @PathVariable Integer id,
//                                        @ApiParam("更新内容") @RequestBody TaDispatch taDispatch) throws Exception{
//
//        if (iTaDispatchService.updateById(taDispatch)){
//            return ResponseBean.success(iTaDispatchService.getById(id));
//        }else {
//            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
//        }
//    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/dispatch/{id}",method= RequestMethod.GET)
    @ApiOperation(value="调度详情", notes = "调度详情", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean getDetail(@ApiParam("调度ID") @PathVariable String id) throws Exception{
        return ResponseBean.success(iTaDispatchService.getExistBy("dispatch_id", id, false, true));
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/order/{orderId}/dispatch",method= RequestMethod.GET)
    @ApiOperation(value="调度详情", notes = "调度详情", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taDispatchGet(@ApiParam("订单ID") @PathVariable String orderId) throws Exception{
        return ResponseBean.success(iTaDispatchService.getExistBy("order_id", orderId, false, true));
    }
}
