package com.yunzhi.nanyang.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.nanyang.common.BaseController;
import com.yunzhi.nanyang.common.Constants;
import com.yunzhi.nanyang.common.ResponseBean;
import com.yunzhi.nanyang.common.StringUtils;
import com.yunzhi.nanyang.entity.*;
import com.yunzhi.nanyang.service.ITaDispatchService;
import com.yunzhi.nanyang.service.ITaOrderService;
import com.yunzhi.nanyang.service.ITaOrgService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import com.yunzhi.nanyang.service.ITaWorkJobService;
import org.springframework.web.bind.annotation.RestController;

import java.time.LocalDateTime;

/**
 * <p>
    * 作业表 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2022-03-08
 */

@Api(tags = "作业表")
@RestController
@RequestMapping("/")
public class TaWorkJobController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaWorkJobController.class);

    @Autowired
    public ITaWorkJobService iTaWorkJobService;

    @Autowired
    public ITaOrderService iTaOrderService;

    @Autowired
    public ITaDispatchService iTaDispatchService;

    @Autowired
    public ITaOrgService iTaOrgService;

    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/{client}/{clientId}/work-job",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taWorkJobList(@ApiParam("客户端") @PathVariable String client,
                                      @ApiParam("客户端ID") @PathVariable String clientId,
                                      @ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                      @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                      @ApiParam("是否我的") @RequestParam(value ="mine", defaultValue = "true") Boolean isMine) throws Exception{
        TaPerson taPerson = currentPerson();
        if (isMine && StringUtils.isEmpty(taPerson.getUserId())) {
            return ResponseBean.error("无权操作");
        }

        String userId = isMine ? taPerson.getUserId() : null;

        IPage<TaWorkJob> pg = new Page<>(pageNum, pageSize);
        IPage<TaWorkJob> result = iTaWorkJobService.getPageBy(pg, userId);
        return ResponseBean.success(result);
    }

    /**
     * 暂停/开启
     * @param id  实体ID
     * @param taWorkJob 实体对象
     * @return
     */
    @RequestMapping(value="/{client}/worker/work-job/{id}/pause",method= RequestMethod.PUT)
    @ApiOperation(value="暂停/开启", notes = "暂停/开启", httpMethod = "PUT", response = ResponseBean.class)
    public ResponseBean taWorkJobUpdate(@ApiParam("客户端") @PathVariable String client,
//                                      @ApiParam("客户端ID") @PathVariable String clientId,
                                        @ApiParam("对象ID") @PathVariable String id,
                                        @ApiParam("是否暂停") @RequestParam("mode") Boolean mode) throws Exception{

        TaPerson taPerson = currentPerson();
        if (StringUtils.isEmpty(taPerson.getUserId())) {
            return ResponseBean.error("无权操作");
        }

        TaWorkJob taWorkJob = iTaWorkJobService.getExistBy("job_id", id, false, true);
        if (!taPerson.getUserId().equals(taWorkJob.getWorkerId())) {
            return ResponseBean.error("无权操作");
        }

        if (Constants.WORK_PAUSE < taWorkJob.getStatus() || Constants.WORK_READY > taWorkJob.getStatus()) {
            return ResponseBean.error("非工作状态, 不能操作");
        }

        int status = mode ? Constants.WORK_DOING : Constants.WORK_PAUSE;

        // 暂停 不影响订单状态
        taWorkJob.setStatus(status);

        if (iTaWorkJobService.updateById(taWorkJob)){
            return ResponseBean.success(taWorkJob);
        }else {
            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 暂停/开启
     * @param id  实体ID
     * @param taWorkJob 实体对象
     * @return
     */
    @RequestMapping(value="/{client}/worker/work-job/{id}/stop",method= RequestMethod.PUT)
    @ApiOperation(value="完成工作", notes = "完成工作", httpMethod = "PUT", response = ResponseBean.class)
    public ResponseBean stopJob(@ApiParam("客户端") @PathVariable String client,
//                              @ApiParam("客户端ID") @PathVariable String clientId,
                                @ApiParam("对象ID") @PathVariable String id,
                                @ApiParam("保存内容") @RequestBody TaWorkJob taWorkJob) throws Exception{

        TaPerson taPerson = currentPerson();
        if (StringUtils.isEmpty(taPerson.getUserId())) {
            return ResponseBean.error("无权操作");
        }

        TaWorkJob origin = iTaWorkJobService.getExistBy("job_id", id, false, true);
        if (origin == null || !taPerson.getUserId().equals(taWorkJob.getWorkerId())) {
            return ResponseBean.error("无权操作");
        }

        if (Constants.WORK_PAUSE != taWorkJob.getStatus() || Constants.WORK_DOING != taWorkJob.getStatus()) {
            return ResponseBean.error("非工作状态, 不能操作");
        }

        // 暂停 不影响订单状态
        origin.setStatus(Constants.WORK_DONE);
        origin.setArea(taWorkJob.getArea());

        if (iTaWorkJobService.updateById(origin)){
            return ResponseBean.success(origin);
        }else {
            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/{client}/{clientId}/work-job/{id}",method= RequestMethod.GET)
    @ApiOperation(value="详情", notes = "详情", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taWorkJobGet(@ApiParam("对象ID") @PathVariable Integer id) throws Exception{
        TaPerson taPerson = currentPerson();
        if (StringUtils.isEmpty(taPerson.getUserId())) {
            return ResponseBean.error("无权操作");
        }

        TaWorkJob taWorkJob = iTaWorkJobService.getExistBy("job_id", id, false, true);

        return ResponseBean.success(taWorkJob);
    }
}
