package com.yunzhi.nanyang.service.impl;

import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.yunzhi.nanyang.common.DateUtils;
import com.yunzhi.nanyang.common.StringUtils;
import com.yunzhi.nanyang.entity.SysUser;
import com.yunzhi.nanyang.entity.TaOrder;
import com.yunzhi.nanyang.entity.TaPerson;
import com.yunzhi.nanyang.mapper.TaOrderMapper;
import com.yunzhi.nanyang.service.ITaOrderService;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;

/**
 * <p>
 * 订单表 服务实现类
 * </p>
 *
 * @author yansen
 * @since 2022-03-08
 */
@Service
public class TaOrderServiceImpl extends BaseServiceImpl<TaOrderMapper, TaOrder> implements ITaOrderService {

    @Override
    public String getNextOrderNO() {
        String today = DateUtils.toString(LocalDateTime.now(), "yyyyMMdd");
        String prefix = "M-" + today;

        int next = baseMapper.getNextOrderNO(prefix);
        return prefix + StringUtils.lpad(String.valueOf(next), "0", 6);
    }

    @Override
    public IPage<TaOrder> getAdminPage(IPage<TaOrder> pg, Boolean isNotDispatched, SysUser sysUser) {
        return baseMapper.getAdminPage(pg, isNotDispatched, null == sysUser ? null : sysUser.getUserId());
    }

    @Override
    public IPage<TaOrder> getClientPage(IPage<TaOrder> pg, TaPerson taPerson, SysUser sysUser, Integer workStatus, String q) {
        String userId = null == sysUser ? null : sysUser.getUserId();
        String personId = null == taPerson ? null : taPerson.getPersonId();

        if (!StringUtils.isEmpty(userId)) {
            return baseMapper.getOrderDispatched(pg, userId, workStatus, q);
        }

        return baseMapper.getClientPage(pg, personId, q);
    }

    @Override
    public boolean updateStatus(String orderId, String statusColumn, int statusValue) {
        UpdateWrapper<TaOrder> updateWrapper = new UpdateWrapper<>();
        updateWrapper.set(statusColumn, statusValue);
        updateWrapper.eq("order_id", orderId);

        return update(updateWrapper);
    }
}
